/*
 * Copyright 1992-1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
//////////////////////////////////////////////////////////////////////
// Curve.h - definition of the curve class
//
// This class defines a curve stretch of road. It is
// derived from the stretch abstract base class.
//////////////////////////////////////////////////////////////////////

#ifndef CURVE_H
#define CURVE_H

#include "Defines.h"

class Stretch;

class Curve : public Stretch {

public:

	Curve(Stretch *prev, Stretch *next, 
		float turn_radius, float width, float turn_angle,
		float bank_angle = 0.0);
		
	~Curve();

protected:

	void make_curve(SbVec3f,SbRotation);

	// override Stretch method
	// for this stretch, at current marker, returns the marker offset
	// for position in the range of 0.0-1.0. If the position is on
	// the next marker, returns TRUE; returns FALSE if on this marker.
	// 0.0 means at the beginning of the segment, 1.0 at the end.
	Boolean get_marker_offset(
		int marker, const SbVec3f position, float &offset);

	// center of rotation of the curve
	SbVec3f _center;

	// angle occupied between markers (in radians)
	float _marker_angle;

};

#endif
	
